package model

type Lbs struct {
	Longitude float64 `json:"longitude"`
	Latitude  float64 `json:"latitude"`
}

type AddressInfo struct {
	Province string `json:"province"`
	City     string `json:"city"`
	County   string `json:"county"`
}

type SemanticReqNativeData struct {
	// TODO: Native API协议数据, 由云端和客户端一起定义
}

type SemanticReqNative struct {
	Name        string                 `json:"name"`
	SeqID       string                 `json:"seqID"`
	Code        int                    `json:"code"`
	Message     string                 `json:"message"`
	DataVersion string                 `json:"dataVersion"`
	Data        *SemanticReqNativeData `json:"data"`
}

type SemanticReqSDKExtend struct {
	CarDOA                int    `json:"carDOA"`
	LastRspType           string `json:"lastRspType"`
	LastRspResult         string `json:"lastRspResult"`
	LastSessionStatus     int    `json:"lastSessionStatus"`
	LastCompleteCmdResult string `json:"lastCompleteCmdResult"`
}

// SemanticReq store data come from client request json.
type SemanticReq struct {
	// CheckStr  string `json:"checkstr,omitempty"`
	// ServiceID string `json:"serviceId,omitempty"`
	// SessionID string `json:"sessionId,omitempty"`
	// AppId     string `json:"appId,omitempty"`
	// //Token     string `json:"token,omitempty"`
	// Mac       string `json:"mac,omitempty"`
	MacWifi  string `json:"macWifi,omitempty"`
	MacVoice string `json:"macVoice,omitempty"`
	// Barcode   string `json:"barcode,omitempty"`
	Query string `json:"query,omitempty"`
	Ip    string `json:"ip,omitempty"`
	// TimeStamp string `json:"timestamp,omitempty"`
	// Classify  string `json:"classify,omitempty"`
	TestID string `json:"testId,omitempty"`
	// //UserID    string `json:"uid,omitempty"`
	// //VerServer string `json:"ver,omitempty"`
	// HomeId      string `json:"homeId,omitempty"`
	// City        string `json:"city,omitempty"`
	// BotKey      string `json:"botKey,omitempty"`
	// Appver      string `json:"appVer,omitempty"`
	// RequestData string `json:"requestBody,omitempty"`
	OriginQuery string `json:"oriQuery,omitempty"`
	// Mode        string `json:"mode"` //是否儿童模式
	Mid string `json:"mid,omitempty"`
	// Version     string `json:"version,omitempty"`
	RequestId   string
	Lbs         *Lbs                  `json:"lbs,omitempty"`    // 经纬度
	Vender      string                `json:"vender,omitempty"` // 细分码
	Appkey      string                `json:"appKey"`
	AccessToken string                `json:"accessToken"`
	Qua         string                `json:"qua"`
	Auth        string                `json:"auth"`
	Dsn         string                `json:"dsn"`
	Guid        string                `json:"guid"`
	Exist       bool                  `json:"exist"`
	TRequestId  string                `json:"tRequestId"`
	RequestType string                `json:"requestType"`
	CommType    string                `json:"commType"`
	CharacterID string                `json:"characterID"`
	Event       string                `json:"event"`
	Nativate    *SemanticReqNative    `json:"nativate"`
	SDKExtend   *SemanticReqSDKExtend `json:"sdkExtend"`
	AddressInfo *AddressInfo          `json:"addressInfo"`
	Language    string                `json:"language"`
}

// Tencent report data
type TencentReport struct {
	UserId     string
	Domain     string
	Intent     string
	ResourceId string
	DataSource string
}

type RicAnswer struct {
	Header struct {
		Guid   string `json:"guid,omitempty"`
		Device struct {
			SerialNum string `json:"serial_num,omitempty"`
		} `json:"device,omitempty"`
		User struct {
			Auth string `json:"authorization,omitempty"`
		} `json:"user"`
		Qua string `json:"qua"`
		Ip  string `json:"ip,omitempty"`
		Lbs *Lbs   `json:"lbs,omitempty"`
	} `json:"header,omitempty"`
	Payload struct {
		Query string `json:"query"`
	} `json:"payload"`
}

type AlarmRemindInfo struct {
	Mac                string
	E_date             string
	E_time             string
	Note               string
	E_type             string
	Repeat_type        int
	Start_time         int
	Status             int
	Last_update_time   int
	Countdown_duration int
	Createtime         string
	Oid                string
	URL                string
	Content            string
	Speech_Type        string
}

// domain 方法的参数
type DomainParams struct {
	JsonData            []byte
	Mac                 string
	Query               string
	Domain              string
	Mid                 string
	TokenSearchResponse *TokenSearchResponse
}

// domain 方法返回的结果
type DomainRes struct {
	JsonByte    []byte
	ReportDatas []TencentReport
}

type DingDangBot struct {
	Key     string
	Secret  string
	UrlType int // url类型, 用于新旧设备的区分
}

type MidAppKey struct {
	Mid         string
	AppKey      string
	AccessToken string
	Vender      string // 细分码
	UrlType     int    // url类型, 用于新旧设备的区分
}

type GreeNluProtocol struct {
	Status struct {
		Code      int    `json:"code"`
		ErrorType string `json:"errorType"`
	} `json:"status"`
	Query    string                  `json:"query"`
	Semantic *map[string]interface{} `json:"semantic"`
	Result   struct {
		Hint       string                  `json:"hint"`
		ResultData *map[string]interface{} `json:"resultData"`
	} `json:"result"`
}

type ParamsStr struct {
	Origin string `json:"orgin"`
	Norm   string `json:"norm"`
	Code   int    `json:"code"`
}

type OutputContextStr struct {
	Context string `json:"context"`
	Service string `json:"service"`
	Class   string `json:"class"`
	Scene   string `json:"scene"`
}

type TokenSearchRequest struct {
	Mac       string `json:"mac"`
	RequestId string `json:"requestId"`
}

type TokenSearchResponse struct {
	Status struct {
		Code int32  `json:"code"`
		Msg  string `json:"msg"`
	} `json:"status"`
	Data struct {
		Dsn           string `json:"dsn"`
		Authorization string `json:"authorization"`
		AccessToken   string `json:"accessToken"`
		AppKey        string `json:"appKey"`
		Status        int32  `json:"status"`
		UriType       int32  `json:"uriType"`
		HomeId        string `json:"homeId"`
	} `json:"data"`
}

// 服务
type Service struct {
	Address string
	Port    int
}

type Location struct {
	Longitude float32 `json:"Longitude"`
	Latitude  float32 `json:"Latitude"`
	City      string  `json:"City,omitempty"`
}

type TencentNlp struct {
	Header struct {
		QUA      string    `json:"QUA"`
		DSN      string    `json:"DSN"`
		GUID     string    `json:"GUID,omitempty"`
		Location *Location `json:"Location,omitempty"`
	} `json:"Header"`
	Payload struct {
		Semantic struct {
			Query string `json:"Query"`
		} `json:"Semantic"`
		RequestType string `json:"RequestType,omitempty"`
		CommType    string `json:"CommType,omitempty"`
	} `json:"Payload"`
}

type DomainParamsV2 struct {
	TencentNlpResp      *TencentNlpResp
	Mac                 string
	Query               string
	Domain              string
	Mid                 string
	MidType             string
	TokenSearchResponse *TokenSearchResponse
	RequestId           string
}

type PayloadDataNative struct {
	Name    string            `json:"Name"`
	SeqID   string            `json:"SeqID"`
	Timeout int               `json:"Timeout"`
	Param   map[string]string `json:"Param"`
}

type PayloadData struct {
	Class  string            `json:"Class"`
	Native PayloadDataNative `json:"Native"`
}

type ResponseBody struct {
	Header struct {
		Semantic struct {
			Code            int                      `json:"code"`
			Domain          string                   `json:"domain"`
			Intent          string                   `json:"intent"`
			Msg             string                   `json:"msg"`
			SessionComplete bool                     `json:"session_complete"`
			SkillId         string                   `json:"skill_id"`
			Slots           []map[string]interface{} `json:"slots,omitempty"`
			Command         interface{}              `json:"Command,omitempty"`
			ResponseType    string                   `json:"ResponseType,omitempty"`
			Data            *PayloadData             `json:"Data,omitempty"`
		} `json:"semantic"`
	} `json:"header"`
	ResponseText string                   `json:"response_text"`
	AsrRecongize string                   `json:"asr_recongize"`
	ListItems    []map[string]interface{} `json:"listItems,omitempty"`
}

type SDKExtend struct {
	CarDOA                int    `json:"CarDOA"`
	LastRspType           string `json:"LastRspType"`
	LastRspResult         string `json:"LastRspResult"`
	LastSessionStatus     int    `json:"LastSessionStatus"`
	LastCompleteCmdResult string `json:"LastCompleteCmdResult"`
}

type Native struct {
	Name        string `json:"Name"`
	SeqID       string `json:"SeqID"`
	Code        int    `json:"Code"`
	Message     string `json:"Message"`
	DataVersion string `json:"DataVersion"`
	Data        struct {
		DataArr []map[string]string `json:"List"`
	} `json:"Data"`
}

// ws req 协议
type TencentNlpWs struct {
	Header struct {
		RequestID string    `json:"RequestID,omitempty"`
		DialogID  string    `json:"DialogID,omitempty"`
		GUID      string    `json:"GUID,omitempty"`
		QUA       string    `json:"QUA,omitempty"`
		AppKey    string    `json:"AppKey,omitempty"`
		DSN       string    `json:"DSN,omitempty"`
		Location  *Location `json:"Location,omitempty"`
	} `json:"Header"`
	Payload struct {
		RequestType string `json:"RequestType,omitempty"`
		CommType    string `json:"CommType,omitempty"`
		Common      struct {
			GUID        string `json:"GUID,omitempty"`
			QUA         string `json:"QUA,omitempty"`
			AppeKey     string `json:"AppeKey,omitempty"`
			DSN         string `json:"DSN,omitempty"`
			CharacterID string `json:"CharacterID,omitempty"`
		} `json:"Common"`
		SDKExtend *SDKExtend `json:"SDKExtend,omitempty"`
		Semantic  struct {
			Query    string `json:"Query,omitempty"`
			Event    string `json:"Event,omitempty"`
			Language string `json:"Language,omitempty"`
		} `json:"Semantic,omitempty"`
		Native *Native `json:"Native,omitempty"`
	} `json:"Payload"`
}

type DomainParamsWs struct {
	TencentNlpResp      *TencentNlpWsResp
	Query               string
	RequestId           string
	Mac                 string
	Domain              string
	Mid                 string
	MidType             string
	TokenSearchResponse *TokenSearchResponse
	SessionId           string
	TencentNlpWs        interface{}
	HomeId              string
	AppKey              string
}

type TencentNluParseStreamMetaData struct {
	MacWifi     string `json:"macWifi,omitempty"`
	MacVoice    string `json:"macVoice,omitempty"`
	Ip          string `json:"ip,omitempty"`
	Mid         string `json:"mid,omitempty"`
	RequestId   string
	Vender      string `json:"vender,omitempty"` // 细分码
	Appkey      string `json:"appKey"`
	AccessToken string `json:"accessToken"`
	Qua         string `json:"qua"`
	Auth        string `json:"auth"`
	Dsn         string `json:"dsn"`
	Exist       bool   `json:"exist"`
	TRequestId  string `json:"tRequestId"`
	SessionId   string `json:"sessionId"`
	HomeId      string `json:"homeId"`
}
